require "import"
import "java.io.FileOutputStream"
import "java.io.OutputStreamWriter"
import "java.io.BufferedReader"
import "java.io.InputStreamReader"
import "java.io.File"
import "java.io.FileReader"
import "java.io.FileWriter"
import "java.io.FileOutputStream"
import "java.io.FileInputStream"
import "android.media.AudioManager"
import "android.speech.tts.UtteranceProgressListener"
import "android.content.Intent"
import "android.content.Context"
import "android.widget.Toast"
import "java.lang.System"
import "android.speech.tts.TextToSpeech"
import "java.util.*"

counterFile = File("/storage/emulated/0/orden.txt")
counterFile.delete()
-- Funciones del contador
local counterFilePath = "/storage/emulated/0/contador.txt"

local function readFileContent(file)
    local content = ""
    local bufferedReader = BufferedReader(InputStreamReader(FileInputStream(file)))
    local line = bufferedReader.readLine()
    while line do
        content = content .. line
        line = bufferedReader.readLine()
    end
    bufferedReader.close()
    return content
end

local function createCounterFile()
    local file = File(counterFilePath)
    if not file.exists() then
        local outputStream = FileOutputStream(file)
        local writer = OutputStreamWriter(outputStream)
        writer.write("0")
        writer.close()
        outputStream.close()
    end
end

local function getCounter()
    createCounterFile()
    local file = File(counterFilePath)
    local content = readFileContent(file)
    local counter = tonumber(content)
    return counter
end

local function updateCounter(counter)
    local file = File(counterFilePath)
    local outputStream = FileOutputStream(file)
    local writer = OutputStreamWriter(outputStream)
    writer.write(tostring(counter))
    writer.close()
    outputStream.close()
end

local function getNextRecordingNumber()
    local counter = getCounter()
    updateCounter(counter + 1)
    return counter
end

-- Configuración TTS y carpeta de salida
folderPath = "/storage/emulated/0/Grabación con Plugin"
folder = File(folderPath)
if not folder.exists() then
    success = folder.mkdirs()
    if success then
        service.asyncSpeak("creado por sergio pina: " .. folderPath)
    else
        service.asyncSpeak("No se pudo crear la carpeta en: " .. folderPath)
    end
end

context = activity or service

local textoIngresado = node.text or ""

local TTS = TextToSpeech(context, function(status)
    if status == TextToSpeech.ERROR then
        service.asyncSpeak("Error al inicializar TextToSpeech")
    else
        service.asyncSpeak("TextToSpeech inicializado exitosamente")
        readAloud()
    end
end)

TTS.setOnUtteranceProgressListener(UtteranceProgressListener{
    onStart = function(utteranceId)
        service.asyncSpeak("Comenzó a procesar: " .. utteranceId)
    end,

    onDone = function(utteranceId)
        service.asyncSpeak("Terminó de procesar: " .. utteranceId)
    end,

    onError = function(utteranceId)
        service.asyncSpeak("Error: " .. utteranceId)
    end
})

-- Divide el texto en fragmentos
local textChunks = {}
local maxChunkSize = 4000

while #textoIngresado > maxChunkSize do
    local chunkEnd = maxChunkSize
    while string.sub(textoIngresado, chunkEnd, chunkEnd) ~= " " and chunkEnd > 1 do
        chunkEnd = chunkEnd - 1
    end
    table.insert(textChunks, string.sub(textoIngresado, 1, chunkEnd))
    textoIngresado = string.sub(textoIngresado, chunkEnd + 1)
end
table.insert(textChunks, textoIngresado) -- Agrega el último fragmento

function readAloud()
    if TTS.isSpeaking() then
        TTS.stop()
        return
    end

    for i, textChunk in ipairs(textChunks) do
        local recordingNumber = getNextRecordingNumber()
        service.asyncSpeak("Procesando fragmento " .. recordingNumber)
        local params = {
            [TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID] = "utteranceId" .. recordingNumber,
            [TextToSpeech.Engine.KEY_PARAM_STREAM] = tostring(AudioManager.STREAM_MUSIC)
        }
        local speakStatus = TTS.synthesizeToFile(textChunk, params, string.format("%s/grabacion_%d.wav", folderPath, recordingNumber))
        if speakStatus == TextToSpeech.ERROR then
            service.asyncSpeak("Error al llamar a TTS.synthesizeToFile para el fragmento " .. recordingNumber)
        end
    end
end

function stopSpeaking()
    if TTS.isSpeaking() then
        TTS.stop()
    end
end

return true